/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/




#include "/lib/universal.glsl"

/* ------ framebuffer attachment outputs ------ */
/*DRAWBUFFERS:012*/
layout(location = 0) out vec4 scenecol_data;
layout(location = 1) out vec4 auxiliary_data1;
layout(location = 2) out vec4 auxiliary_data2;

/* ------ inputs from vertex stage ------ */
in mat2x2 coord;

flat in int mat_id;

flat in vec3 normal;

in vec4 tint;

#if defined g_textured && defined normals_resourcepack
    flat in mat3x3 tbn;
#endif

#ifdef g_terrain
    flat in int foliage;

    #ifndef specular_resourcepack
        flat in ivec4 pbr_val;
    #endif
#endif

/* ------ uniforms ------ */
#ifdef g_textured
    uniform sampler2D gcolor;

    #ifdef specular_resourcepack
        uniform sampler2D specular;
    #endif

    #ifdef normals_resourcepack
        uniform sampler2D normals;
    #endif

    #ifdef g_entities
        uniform vec4 entityColor;
    #endif
#endif

/* ------ includes ------ */
#include "/lib/util/srgb.glsl"
#include "/lib/util/encoders.glsl"

#if defined g_textured && defined normals_resourcepack
    #include "/lib/fragment/labPBR.glsl"
#endif

void main() {
    #ifdef hide_terrain
        discard;
    #endif

    #ifdef g_textured
        scenecol_data            = gtex(gcolor);      //scenecol_data would always refer to the main scenecolor that is visible to the player
        if(scenecol_data.a < 0.1) discard;
        scenecol_data.rgb       *= tint.rgb;
        
        vec3 scenenormal         = normal;

        #ifdef g_entities
            scenecol_data.rgb    = mix(scenecol_data.rgb, entityColor.rgb, entityColor.a);
        #endif

        #ifdef white_world
            scenecol_data.rgb    = vec3(1.0);
        #endif

        #ifdef g_terrain
            scenecol_data.a      = tint.a;   //override that with the vanilla ao
        #else
            scenecol_data.a      = 1.0;      //it isn't terrain, it doesn't have ao
        #endif
        
        #ifdef specular_resourcepack
            vec4 spectex         = gtex(specular);
            vec2 specularData    = vec2(encode2x8(spectex.rg), encode2x8(spectex.ba));

            #ifdef normals_resourcepack
                vec4 ntex        = gtex(normals);
                scenenormal      = decode_lab_nrm(ntex.rgb, tbn, scenecol_data.a);
            #endif
        #else
            #ifdef g_terrain
                vec4 specdata    = vec4(pbr_val)/255.0;
            #else
                vec4 specdata    = vec4(0.0);
            #endif

            #ifdef normals_resourcepack
                vec4 ntex        = gtex(normals);
                scenenormal      = decode_nrm(ntex.rgb, tbn);
            #endif
            
            vec2 specularData       = vec2(encode2x8(specdata.rg), encode2x8(specdata.ba));
        #endif
    #else
        scenecol_data            = tint;
        if(scenecol_data.a < 0.1) discard;
        scenecol_data.a          = 1.0;  //i don't see how there should be vanilla ao on non-terrain stuff so let's override this
        
        vec3 scenenormal         = normal;

        const vec2 specularData  = vec2(0.0);
    #endif

    scenecol_data                = make_drawbuffer(SRGBtoACEScgAlbedo(scenecol_data.rgb), saturate(scenecol_data.a));
    auxiliary_data1              = vec4(encodeNormal(scenenormal), encode2x8(coord[1]), specularData.y);
    auxiliary_data2              = vec4(encodeMatID16(mat_id), 0.0, specularData.x, 1.0);
}