/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/



#include "/lib/universal.glsl"

/* ------ outputs ------ */
out mat2x2 coord;   //0-texcoord, 1-lightmap

flat out int mat_id;

flat out vec3 normal;

out vec4 tint;

#if defined g_textured && defined normals_resourcepack
    flat out mat3x3 tbn;
#endif

#ifdef g_terrain
    flat out int foliage;

    #ifndef specular_resourcepack
        flat out ivec4 pbr_val; //x-smoothness, y-f0, z-stuff, a-emission, is used whithout resourcepack support, is done in bits
    #endif
#endif

/* ------ uniforms & attributes ------ */
uniform vec2 taaOffset;

uniform vec3 sunVector, sunVectorView;
uniform vec3 moonVector, moonVectorView;
uniform vec3 sunPosition, upPosition;

uniform mat4 gbufferModelViewInverse;

#if defined g_textured && defined normals_resourcepack
    attribute vec4 at_tangent;
#endif

#ifdef g_terrain
    attribute vec4 mc_Entity;

    #ifdef waving_foliage_enabled
        #ifdef dynamic_wind_direction
            uniform float windRotation;
        #endif
    
        uniform float frameTimeCounter;
        uniform float rainStrength;
        
        uniform vec3 cameraPosition;
        
        uniform mat4 gbufferModelView;
        
        uniform sampler2D noisetex;
        
        attribute vec4 mc_midTexCoord;
    #endif
#endif

/* ------ includes ------ */
#ifdef g_terrain
    #include "/lib/fragment/labPBR.glsl"

    #ifdef waving_foliage_enabled
        #include "/program/common/foliage.glsl"
    #endif
#endif

void main() {
    coord[0] = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
    coord[1] = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    coord[1].y = linStep(coord[1].y, 1.0/16.0, 1.0);
    normal 	= mat3(gbufferModelViewInverse) * normalize(gl_NormalMatrix*gl_Normal);
    tint    = gl_Color;

    vec4 pos = gl_Vertex;

    pos = transMAD(gl_ModelViewMatrix, pos.xyz).xyzz;

    #if defined g_terrain && defined waving_foliage_enabled
        pos = transMAD(gbufferModelViewInverse, pos.xyz).xyzz;
        if(length(pos.xz) < wf_lod) pos.xyz = windFoliage(pos.xyz);
        pos = transMAD(gbufferModelView, pos.xyz).xyzz;
    #endif

    gl_Position = pos.xyzz * diagonal4(gl_ProjectionMatrix) + vec4(0, 0, gl_ProjectionMatrix[3].z, 0);

    #ifdef taa_enabled
    gl_Position.xy += taaOffset * gl_Position.w;
    #endif

    mat_id = 1;

    #ifdef g_textured
        #ifndef g_terrain
        mat_id = 2;
        #endif

        #ifdef g_entities
        mat_id = 1;
        #endif

        #ifdef normals_resourcepack
            vec3 viewtangent = normalize(gl_NormalMatrix*at_tangent.xyz);
            vec3 viewbinormal = normalize(gl_NormalMatrix*cross(at_tangent.xyz, gl_Normal.xyz) * at_tangent.w);
            vec3 tangent = mat3(gbufferModelViewInverse) * viewtangent;
            vec3 binormal = mat3(gbufferModelViewInverse) * viewbinormal;

            tbn     = mat3(tangent.x, binormal.x, normal.x,
                        tangent.y, binormal.y, normal.y,
                        tangent.z, binormal.z, normal.z);
        #endif
    #endif

    #ifdef g_terrain
        if (mc_Entity.x == 10021) mat_id = 4;

        if (mc_Entity.x == 10022 ||
         mc_Entity.x == 10023 ||
         mc_Entity.x == 10024 ||
         mc_Entity.x == 10025 ||
         mc_Entity.x == 10026) mat_id = 3;

        #ifndef specular_resourcepack
            #ifdef hardcoded_specular
                if (mc_Entity.x == 10101) {
                    pbr_val = ivec4(190, 230, 0, 0);
                } else if (mc_Entity.x == 10102) {
                    pbr_val = ivec4(190, 231, 0, 0);
                } else if (mc_Entity.x == 10104) {
                    pbr_val = ivec4(230, lab_f0(0.018), 0, 0);
                } else {
                    pbr_val = ivec4(0, 0, 20, 0);
                }
            #else 
                pbr_val = ivec4(0, 0, 20, 0);
            #endif
        #endif
    #endif
}