/* ------ settings ------ */
#define vcloud_quality 1.0      //[0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.2 1.4 1.6 1.8 2.0 2.2 2.4 2.6 2.8 3.0]
#define vcloud_direct_steps 6   //[4 5 6 7 8 9 10 12 16 20 24]
#define vcloud_skylight_steps 4 //[2 3 4 5 6 7 8 12 16 20 24]
#define vcloud_thresh 0.075     //[0.2 0.15 0.1 0.075 0.05 0.025 0.01]
#define vcloud_shapeq 1         //[0 1 2]

#define cloudLayer0Mode
#define cloudLayer1Mode
#define cloudLayer2Mode 1      //[0 1 2]
#define cloudLayer3Mode

#define cloudLayer0TypeOverride 0  //[-2 -1.0 -0.75 -0.5 -0.25 0 0.25 0.5 0.75 1.0 2]
#define cloudLayer1TypeOverride 0  //[-2 -1.0 -0.75 -0.5 -0.25 0 0.25 0.5 0.75 1.0 2]
#define cloudLayer2TypeOverride 0  //[-2 -1.0 -0.75 -0.5 -0.25 0 0.25 0.5 0.75 1.0 2]

const float vcloud_clip     = 110e3;

/* ------ constants for cloud types ------ */
/*
CLOUD TYPE LIST:

Layer 0 (Volumetric, 500-3000m), using a single, shared shape function:
Cumulus mediocris (Medium-Large cumuli)
Cumulus humilis (small fair-weather cumulus, shares shape with mediocris)
Stratocumulus (Medium-Large but flatter cumulus, 500-2000m)

Layer 1 (Volumetric, 3000-7000m), separate shape functions:
Altocumulus (small cumulus-like clouds)
Altostratus undulatus (large, sheet-like stratus)

Layer 2 (Volumetric/Planar, 7000-11000m), separate shape functions:
Cirrocumulus floccus (small cumulus-like clouds, very low density)
Cirrus uncinus (medium cirrus clouds, low density and little shading)

Layer 3 (Planar, 11000-14000m)
Cirrostratus (very thin uniform cloud, no shading needed)

Layer 4 (Planar, 76-85km)
Noctulicent clouds
*/

const float clayer0_minY    = 5e2;
const float clayer0_maxY    = 3e3;
const float clayer1_maxY    = 7e3;
const float clayer2_maxY    = 11e3;
const float clayer3_maxY    = 14e3;

const float cloudVolumeMinY = clayer0_minY;
#if (cloudLayer2Mode == 2 || (cloudLayer2Mode != 0 && defined PD_REFLECTION))
    const float cloudVolumeMaxY = clayer2_maxY;
#else
    const float cloudVolumeMaxY = clayer1_maxY;
#endif
const float cloudVolumeTotalDepth = cloudVolumeMaxY - cloudVolumeMinY;

#define clayer0_size 14e-4
#define clayer1_size 7e-4
#define clayer2_size 5e-4
#define clayer3_size 7e-5

#define cumulusMaxY 3e3
#define stratocumulusMaxY 2e3

#define altocumulusMinY 3.3e3
#define altocumulusMaxY 5e3
#define altostratusMinY 4e3
#define altostratusMaxY 6.6e3

#define cirrocumulusMinY 7.5e3
#define cirrocumulusMaxY 8.5e3
#define cirrusMinY 9e3
#define cirrusMaxY 11e3

#define cloudLayer2PlanarMinY 7.5e3
#define cloudLayer2PlanarMaxY 10e3
#define cloudLayer2PlaneY 8.65e3
const float cloudLayer2PlaneDepth = cloudLayer2PlanarMaxY - cloudLayer2PlanarMinY;

#define cirrostratusMinY 11e3
#define cirrostratusMaxY 14e3
const float cloudLayer3PlaneDepth = cirrostratusMaxY - cirrostratusMinY;
const float cloudLayer3PlaneY = cirrostratusMinY + cloudLayer3PlaneDepth * 0.35;

#define clayer0CovBias 0.0  //[-0.5 -0.4 -0.3 -0.2 -0.1 0.0 0.1 0.2 0.3 0.4 0.5]
#define clayer1CovBias 0.0  //[-0.5 -0.4 -0.3 -0.2 -0.1 0.0 0.1 0.2 0.3 0.4 0.5]
#define clayer2CovBias 0.0  //[-0.5 -0.4 -0.3 -0.2 -0.1 0.0 0.1 0.2 0.3 0.4 0.5]
#define clayer3CovBias 0.0  //[-0.5 -0.4 -0.3 -0.2 -0.1 0.0 0.1 0.2 0.3 0.4 0.5]