float cloudTick    = frameTimeCounter * 1e-1;

#define readNoise2D(x) texture(noisetex, (x))

#include "cloudConstants.glsl"


/* ------ phase function ------ */
#ifdef PD_CLOUDRENDER
float cloud_mie_dumb(float cos_theta, float g) {
    float sqG   = sqr(g);
    float a     = (1.0-sqG) / (2.0 + sqG);
    float b     = (1.0 + sqr(cos_theta)) / (1.0 + sqG - 2.0*g*cos_theta);

    return max(1.5 * a*b + g*cos_theta, 0.0)*rcp(pi);
}
float cloud_mie(float x, float g) {
    float mie   = 1.0 + sqr(g) - 2.0*g*x;
        mie     = (1.0 - sqr(g)) / ((4.0*pi) * mie*(mie*0.5+0.5));
    return mie;
}
float cloud_phase(float cos_theta, float g) {
    float a     = cloud_mie(cos_theta, 0.44*g)+cloud_mie_dumb(cos_theta, 0.89*g)*0.75;
    float b     = cloud_mie(cos_theta, -0.25*g)*1.4*cloudBackscattering;

    return mix(a, b, 0.38);   //kinda looks better to me with a lil min value
}
#endif


/* ------ Layer 0 Shape ------ */
uniform vec4 cloudLayer0data;
vec3 cloudLayer0Wind    = normalize(vec3(0.55, -0.05, 0.45)) * cloudTick;
float cloudLayer0(vec3 pos, float altitude) {
    //return 0.0;
    pos    *= clayer0_size;
    pos    += cloudLayer0Wind * 1e-1;

    vec4 typeNoise      = readNoise2D((pos.xz + cloudLayer0Wind.xz * 3e-2) * 3e-3);

    float stratoFade    = sstep(typeNoise.b + cloudLayer0data.w + cloudLayer0TypeOverride, 0.4, 0.6);

    //return stratoFade;
        //stratoFade = 0.5;

    float maxalt        = mix(cumulusMaxY, stratocumulusMaxY, stratoFade);
    const float minalt  = clayer0_minY;

    if (altitude > maxalt || altitude < minalt) return 0.0;

    float depth         = maxalt - minalt;


    //  coverage section

    float erodeLow      = 1.0 - sstep(altitude, minalt, minalt + (depth * 0.13));
    float erodeHigh     = sstep(altitude, minalt + (depth * 0.16), maxalt);
    float fadeLow       = sstep(altitude, minalt, minalt + (depth * 0.075));
    float fadeHigh      = 1.0 - sstep(altitude, minalt + (depth * 0.65), maxalt);

    //if (max(fadeLow, fadeHigh) < 1e-2) return 0.0;

    float localCoverage = readNoise2D((pos.xz + cloudLayer0Wind.xz * 5e-2) * 8e-4).x;
        localCoverage   = sstep(localCoverage - cloudLayer0data.y, 0.33, 0.8);
        //localCoverage   = 0.0;

    //return localCoverage;

    if (localCoverage > 0.995) return 0.0;

    float coverageBias  = (stratoFade * 0.0) + 0.41 - clayer0CovBias;

    vec4 nCov           = readNoise2D(pos.xz * 3e-2);

    float coverage      = mix(nCov.z, typeNoise.x, stratoFade * 0.24) + cloudLayer0data.x;
        coverage        = (coverage - coverageBias) * rcp(saturate(1.0 - coverageBias)) - (localCoverage * 0.5);
        coverage       *= (1.0  - localCoverage);

        coverage        = (coverage * fadeLow) - (erodeLow * (0.2 + stratoFade * 0.1));
        coverage        = (coverage * fadeHigh) - (erodeHigh * (0.75 + localCoverage * 0.1));
        coverage        = saturate(coverage * 1.25);
    
    if (coverage < 36e-3) return 0.0;


    //  shape detail section

    float shape         = coverage;
    float slope         = sqrt(saturate(1.0 - shape));

    float fadeTop       = sstep(altitude, minalt + (depth * 0.2), minalt + (depth * 0.65));

    float fadeDensity   = (sstep(altitude, minalt, minalt + (depth * 0.2)) * 0.08) + 1e-2;
        fadeDensity    += (sstep(altitude, minalt + (depth * 0.1), minalt + (depth * 0.55)) * 0.3) + (fadeTop * 1.2);
        fadeDensity    += sstep(altitude, minalt + (depth * 0.4), maxalt - (depth * 0.15));
        fadeDensity    *= 1.5 + stratoFade * 0.5;

    if (slope > 1e-8) {
        float fadeWorley = sstep(altitude, minalt, minalt + (depth * 0.33)) * 0.5 + 0.5;

        vec3 n3D        = noise_3d(pos * 0.45);
        vec2 nd         = vec2(n3D.x + n3D.y * 0.3, 1.3);

            shape      -= (1.0 - (nd.x * rcp(nd.y))) * 0.35 * fadeWorley * ((1.0 + stratoFade * 0.2) * slope);
        
        if (shape < 0.0) return 0.0;

            slope       = sqrt(saturate(1.0 - shape));

            pos.xy     += shape * 0.45;

            shape      -= value_3d(pos * 32.0) * 0.1 * slope; 
    }

    return max(shape * fadeDensity * cloudLayer0data.z, 0.0);
}

/* ------ Layer 1 Shapes ------ */
uniform vec4 cloudLayer1data;
vec3 cloudLayer1Wind    = normalize(vec3(0.55, -0.15, 0.25)) * cloudTick;
float cloudLayer1Altocumulus(vec3 pos, float altitude, float coverageFade) {
    const float maxalt  = altocumulusMaxY;
    const float minalt  = altocumulusMinY;

    if (altitude > maxalt || altitude < minalt) return 0.0;

    const float depth   = maxalt - minalt;


    //  coverage section

    float erodeLow      = 1.0 - sstep(altitude, minalt, minalt + (depth * 0.25));
    float erodeHigh     = sstep(altitude, minalt + (depth * 0.3), maxalt);
    float fadeLow       = sstep(altitude, minalt, minalt + (depth * 0.15));
    float fadeHigh      = 1.0 - sstep(altitude, minalt + (depth * 0.6), maxalt);

    //if (max(fadeLow, fadeHigh) < 1e-2) return 0.0;

    float localCoverage = readNoise2D((pos.xz + cloudLayer1Wind.xz * 3e-2) * 10e-4 - 0.325).x;
        localCoverage   = sstep(localCoverage - cloudLayer1data.y, 0.33, 0.8);

    //return localCoverage;

    if (localCoverage > 0.995) return 0.0;

    const float coverageBias  = 0.41 - (clayer1CovBias * 0.8);

    vec4 nCov           = readNoise2D(pos.xz * 6e-2);

    float coverage      = nCov.z + cloudLayer1data.x;
        coverage        = (coverage - coverageBias) * rcp(saturate(1.0 - coverageBias)) - (localCoverage * 0.5);
        coverage       *= (1.0  - localCoverage);
        coverage        = (coverage - (coverageFade * 0.3)) * (1.0 - coverageFade);

        coverage        = (coverage * fadeLow) - (erodeLow * 0.4);
        coverage        = (coverage * fadeHigh) - (erodeHigh * (0.75 + localCoverage * 0.1));
        coverage        = saturate(coverage * 1.25);
    
    if (coverage < 36e-3) return 0.0;


    //  shape detail section

    float shape         = coverage;
    float slope         = sqrt(saturate(1.0 - shape));

    float fadeDensity   = (sstep(altitude, minalt, minalt + (depth * 0.2)) * 0.08) + 1e-2;
        fadeDensity    += (sstep(altitude, minalt + (depth * 0.1), minalt + (depth * 0.55)) * 0.3);
        fadeDensity    += sstep(altitude, minalt + (depth * 0.4), maxalt - (depth * 0.15));

    if (slope > 1e-8) {
        float fadeWorley = sstep(altitude, minalt, minalt + (depth * 0.33)) * 0.5 + 0.5;

        vec3 n3D        = noise_3d(pos * 0.5);
        vec2 nd         = vec2(n3D.x + n3D.y * 0.5, 1.5);

            shape      -= (1.0 - (nd.x * rcp(nd.y))) * 0.5 * slope;
        
        if (shape < 0.0) return 0.0;

            slope       = sqrt(saturate(1.0 - shape));

        if (slope > 1e-8) {
            pos.xy     -= shape * 0.45;

            shape      -= value_3d(pos * 32.0) * 0.15 * slope;
        }
    }

    return max(shape * fadeDensity * cloudLayer1data.z, 0.0);
}
float cloudLayer1Altostratus(vec3 pos, float altitude, float coverageFade) {
    const float maxalt  = altostratusMaxY;
    const float minalt  = altostratusMinY;

    if (altitude > maxalt || altitude < minalt) return 0.0;

    const float depth   = maxalt - minalt;


    //  coverage section

    float erodeLow      = 1.0 - sstep(altitude, minalt, minalt + (depth * 0.4));
    float erodeHigh     = sstep(altitude, minalt + (depth * 0.45), maxalt);
    float fadeLow       = sstep(altitude, minalt, minalt + (depth * 0.25));
    float fadeHigh      = 1.0 - sstep(altitude, minalt + (depth * 0.65), maxalt);

    //if (max(fadeLow, fadeHigh) < 1e-2) return 0.0;

    float localCoverage = readNoise2D((pos.xz + cloudLayer1Wind.xz * 3e-2) * 4e-4 - 0.365).x;
        localCoverage   = sqr(sstep(localCoverage - cloudLayer1data.y, 0.3, 0.8));

    //return localCoverage;

    if (localCoverage > 0.995) return 0.0;

    const float coverageBias  = 0.21 - (clayer1CovBias * 0.4);

    vec4 nCov           = readNoise2D((pos.xz * vec2(1.0, 0.42)) * 4e-2 + 0.045);

    float coverage      = nCov.z + cloudLayer1data.x;
        coverage        = (coverage - coverageBias) * rcp(saturate(1.0 - coverageBias)) - (localCoverage * 0.5);
        coverage       *= (1.0  - localCoverage);
        coverage        = (coverage - (coverageFade * 0.4)) * (1.0 - coverageFade);

        coverage        = (coverage * fadeLow) - (erodeLow * 0.6);
        coverage        = (coverage * fadeHigh) - (erodeHigh * (0.75));
        coverage        = saturate(coverage * 1.25);
    
    if (coverage < 36e-3) return 0.0;


    //  shape detail section

    float shape         = coverage;
    float slope         = sqrt(saturate(1.0 - shape));

    float fadeDensity   = (sstep(altitude, minalt, minalt + (depth * 0.2)) * 0.08) + 1e-2;
        fadeDensity    += (sstep(altitude, minalt + (depth * 0.1), minalt + (depth * 0.4)) * 0.3);
        //fadeDensity    += sstep(altitude, minalt + (depth * 0.4), maxalt - (depth * 0.15));

    if (slope > 1e-8) {
            pos.z      *= 0.6;

            pos.xz     += shape * 0.7;

            shape      -= value_3d(pos * 8.0) * 0.3 * slope; 
        
        if (shape < 0.0) return 0.0;

            slope       = sqrt(saturate(1.0 - shape));

        //if (slope > 1e-8) {
            pos.xy     -= shape * 0.45;

            shape      -= value_3d(pos * 24.0) * 0.1 * slope;
        //}
    }

    return max(shape * fadeDensity * 0.3 * cloudLayer1data.z, 0.0);
}
vec4 cloudLayer1(vec3 pos, float altitude) {
    //return vec4(0.0);
    pos    *= clayer1_size;
    pos    += cloudLayer1Wind * 12e-2;

    float typeNoise     = readNoise2D(pos.xz * 1e-3 + 0.681).b + cloudLayer1data.w + cloudLayer1TypeOverride;
    //typeNoise = 1;

    float coverageFade  = (linStep(typeNoise, 0.45, 0.54) * (1.0 - linStep(typeNoise, 0.56, 0.65)));

    if (typeNoise > 0.55) return vec4(cloudLayer1Altostratus(pos, altitude, coverageFade), 23.0, 2.0, 0.76);
    else return vec4(cloudLayer1Altocumulus(pos, altitude, coverageFade), 30.0, 2.0, 0.8);
}

/* ------ Layer 2 Shapes ------ */
uniform vec4 cloudLayer2data;
#if (cloudLayer2Mode == 2 && !(defined PD_REFLECTION))
    #include "cloudLayer2Volume.glsl"
#else
    #include "cloudLayer2Planar.glsl"
#endif

float cloudLayer2(vec3 pos, float altitude) {
    //return 0.0;
    pos    *= clayer2_size;
    pos    += cloudLayer2Wind * 12e-2;

    vec4 n  = readNoise2D(pos.xz * 9e-4 + 0.249);

    float typeNoise     = n.b + cloudLayer2data.w + cloudLayer2TypeOverride;
    //typeNoise = 0.0;

    float coverageFade  = (linStep(typeNoise, 0.47, 0.54) * (1.0 - linStep(typeNoise, 0.56, 0.63)));

    if (typeNoise > 0.55) return cloudLayer2Cirrus(pos, altitude, coverageFade, n.x);
    else return cloudLayer2Cirrocumulus(pos, altitude, coverageFade);
}

/* ------ Layer 3 Shape ------ */
uniform vec2 cloudLayer3data;
vec3 cloudLayer3Wind    = normalize(vec3(1.0, 0.0, 0.4)) * cloudTick;
float cloudLayer3(vec3 pos, float altitude) {   //cirrostratus
    const float maxalt  = cirrostratusMaxY;
    const float minalt  = cirrostratusMinY;

    if (altitude > maxalt || altitude < minalt) return 0.0;

    const float depth   = maxalt - minalt;

    pos    *= clayer3_size;
    pos    += cloudLayer3Wind * 8e-2;


    //  coverage section

    float erodeLow      = 1.0 - sstep(altitude, minalt, minalt + (depth * 0.25));
    float erodeHigh     = sstep(altitude, minalt + (depth * 0.6), maxalt);
    float fadeLow       = sstep(altitude, minalt, minalt + (depth * 0.33));
    float fadeHigh      = 1.0 - sstep(altitude, minalt + (depth * 0.7), maxalt);

    //if (max(fadeLow, fadeHigh) < 1e-2) return 0.0;

    pos.xz     += texture(noisetex, (pos.xz - cloudLayer3Wind.xz * 0.05) * 0.004 * 8.0).z * vec2(1.5, 1.0);

    pos.x  *= 0.5;

    //return localCoverage;

    const float coverageBias  = 0.43;

    vec4 nCov           = readNoise2D(pos.xz * 26e-3 + 0.15);

    float coverage      = nCov.z + cloudLayer3data.x;
        coverage        = (coverage - coverageBias) * rcp(saturate(1.0 - coverageBias));

        coverage        = (coverage * fadeLow) - (erodeLow * 0.6);
        coverage        = (coverage * fadeHigh) - (erodeHigh * (0.5));
    
    if (coverage < 36e-3) return 0.0;

    //return coverage;


    //  shape detail section

    float shape         = coverage;
    float slope         = sqrt(saturate(1.0 - shape));

    if (slope > 1e-8) {
            //pos.z      *= 0.6;

            pos.xy     += shape * 1.2;

        float n         = value_3d(pos * 6.0) * 0.3;

            shape      -= n * slope; pos -= n;
        
        if (shape < 0.0) return 0.0;

            slope       = sqrt(saturate(1.0 - shape));

        if (slope > 1e-8) {

            shape      -= value_3d(pos * 48.0 * vec3(0.4, 1.0, 1.0)) * 0.08 * slope;

            shape *= max(shape, 0.0);
        }
    }

    return max(shape * 0.16 * cloudLayer3data.y, 0.0);
}


float volumeCloudContainer(vec3 pos, inout vec3 powderParam) {
    //return 0.0;
    if (pos.y < clayer0_maxY) {
        #ifdef cloudLayer0Mode
            return cloudLayer0(pos, pos.y);
        #else
            return 0.0;
        #endif
    } else if (pos.y < clayer1_maxY) {
        #ifdef cloudLayer1Mode
            vec4 t = cloudLayer1(pos, pos.y);
            powderParam = t.yzw;
            return t.x;
        #else
            return 0.0;
        #endif
    }
    #if (cloudLayer2Mode == 2 && !(defined PD_REFLECTION))
        else if (pos.y < clayer2_maxY) {
            powderParam     = vec3(40.0, 2.0, 0.65);
            return cloudLayer2(pos, pos.y);
        }
    #endif

    return 0.0;
}

float volumeCloudContainer(vec3 pos) {
    //return 0.0;
    if (pos.y < clayer0_maxY) {
        #ifdef cloudLayer0Mode
            return cloudLayer0(pos, pos.y);
        #else
            return 0.0;
        #endif
    }
    else if (pos.y < clayer1_maxY) {
        #ifdef cloudLayer1Mode
            return cloudLayer1(pos, pos.y).x;
        #endif
    }
    #if (cloudLayer2Mode == 2 && !(defined PD_REFLECTION))
        else if (pos.y < clayer2_maxY) {
            return cloudLayer2(pos, pos.y);
        }
    #endif

    return 0.0;
}


/* ------ lighting functions ------ */
float volumeCloudDirectOD(in vec3 pos, const int steps) {
    const float basestep    = (22.2 * 6.0) / steps;
    const float exponent    = 2.2;

    float stepsize  = basestep;
    float prev_step = stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += cloudLightvector * stepsize) {
        if(pos.y > cloudVolumeMaxY || pos.y < cloudVolumeMinY) break;

        prev_step = stepsize;
        stepsize *= exponent;
        stepsize  = min(stepsize, cloudVolumeTotalDepth / steps);

        float density = volumeCloudContainer(pos);
        if (density <= 0.0) continue;
        od += density*prev_step;
    } 

    return od;
}
float volumeCloudDirectOD(in vec3 pos, const int steps, float noise) {
    const float basestep    = (22.2 * 6.0) / steps;
    const float exponent    = 2.2;

    float stepsize  = basestep;
    float prev_step = stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += cloudLightvector * stepsize) {
        if(pos.y > cloudVolumeMaxY || pos.y < cloudVolumeMinY) break;

        prev_step = stepsize;
        stepsize *= exponent;
        stepsize  = min(stepsize, cloudVolumeTotalDepth / steps);

        float density = volumeCloudContainer(pos + (noise * stepsize) * cloudLightvector);
        if (density <= 0.0) continue;
        od += density*prev_step;
    } 

    return od;
}

float volumeCloudSkyOD(in vec3 pos, const int steps) {
    const vec3 direction = vec3(0, 1, 0);

    float stepsize = cloudVolumeTotalDepth / steps;
        stepsize  *= 1.0 - linStep(pos.y, cloudVolumeMinY, cloudVolumeMaxY)*0.9;

    vec3 increment = direction * stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += increment) {
        if(pos.y > cloudVolumeMaxY) break;

        float density = volumeCloudContainer(pos);
        if (density <= 0.0) continue;
        od += density*stepsize;
    } 

    return od;
}
float volumeCloudSkyOD(in vec3 pos, const int steps, float noise) {
    const vec3 direction = vec3(0, 1, 0);

    float stepsize = cloudVolumeTotalDepth / steps;
        stepsize  *= 1.0 - linStep(pos.y, cloudVolumeMinY, cloudVolumeMaxY)*0.9;

    vec3 increment = direction * stepsize;

        pos       += direction * noise * stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += increment) {
        if(pos.y > cloudVolumeMaxY) break;

        float density = volumeCloudContainer(pos);
        if (density <= 0.0) continue;
        od += density*stepsize;
    } 

    return od;
}

float volumeCloudBouncedLight(vec3 pos) {
    return pow3(1.0-sstep(pos.y, cloudVolumeMinY, cloudVolumeMinY + 850.0));
}

float planarCloudL2DirectOD(in vec3 pos, const int steps) {
    const float basestep    = (20.0 * 6.0) / steps;
    const float exponent    = 2.2;

    float stepsize  = basestep;
    float prev_step = stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += cloudLightvector * stepsize) {
        if(pos.y > cloudLayer2PlanarMaxY || pos.y < cloudLayer2PlanarMinY) break;

        prev_step = stepsize;
        stepsize *= exponent;

        float density = cloudLayer2(pos, pos.y);
        if (density <= 0.0) continue;
        od += density*prev_step;
    } 

    return od;
}
float planarCloudL2SkyOD(in vec3 pos, const int steps) {
    const vec3 direction = vec3(0, 1, 0);

    float stepsize = cloudLayer2PlaneDepth / steps;
        stepsize  *= 1.0 - linStep(pos.y, cloudLayer2PlanarMinY, cloudLayer2PlanarMaxY)*0.9;

    vec3 increment = direction * stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += increment) {
        if(pos.y > cloudLayer2PlanarMaxY) break;

        float density = cloudLayer2(pos, pos.y);
        if (density <= 0.0) continue;
        od += density*stepsize;
    } 

    return od;
}

float planarCloudL3DirectOD(in vec3 pos, const int steps) {
    const float basestep    = (30.0 * 6.0) / steps;
    const float exponent    = 2.4;

    float stepsize  = basestep;
    float prev_step = stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += cloudLightvector * stepsize) {
        if(pos.y > cirrostratusMaxY || pos.y < cirrostratusMinY) break;

        prev_step = stepsize;
        stepsize *= exponent;

        float density = cloudLayer3(pos, pos.y);
        if (density <= 0.0) continue;
        od += density*prev_step;
    } 

    return od;
}
float planarCloudL3SkyOD(in vec3 pos, const int steps) {
    const vec3 direction = vec3(0, 1, 0);

    float stepsize = cloudLayer3PlaneDepth / steps;
        stepsize  *= 1.0 - linStep(pos.y, cirrostratusMinY, cirrostratusMaxY)*0.9;

    vec3 increment = direction * stepsize;

    float od = 0.0;
    for(int i = 0; i < steps; ++i, pos += increment) {
        if(pos.y > cirrostratusMaxY) break;

        float density = cloudLayer3(pos, pos.y);
        if (density <= 0.0) continue;
        od += density*stepsize;
    } 

    return od;
}