
/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

#ifdef atmos_dynamic
uniform vec3 atmos_density_mult;
#else
const vec3 atmos_density_mult = vec3(1.0);
#endif

vec3 calculateAtmosphereDensity(in float h) {
    vec2 rm = expf(-h * inverseScaleHeights + scaledPlanetRadius);

    float o = expf(-max(0.0, (38000.0 - h) - atmosphereRadius) * rcp(8500.0))
	            * expf(-max(0.0, (h - 38000.0) - atmosphereRadius) * rcp(18500.0));

    return vec3(rm, o) * atmos_density_mult;
}

vec3 calculateAtmosphereTransmittance(in vec3 rayVector, in vec3 position, in mat3x3 ec, const int steps) {

    vec2 atmosphereDists = rsi(position, rayVector, atmosphereRadius);
    vec2 planetDists = rsi(position, rayVector, atmosphereLowerLimit);

	bool planetIntersected = planetDists.y >= 0.0;
   	bool atmosphereIntersected = atmosphereDists.y >= 0.0;

    vec2 sd = vec2(0.0);

    if (planetIntersected && planetDists.x < 0.0) sd.x  = planetDists.y;
    else sd.x  = max(atmosphereDists.x, 0.0);

    if (planetIntersected && planetDists.x > 0.0) sd.y  = planetDists.x;
    else sd.y  = atmosphereDists.y;

	float stepSize = length(sd.y - sd.x) * rcp(steps);
	vec3  increment = rayVector * stepSize;
	position += rayVector * sd.x + (increment * 0.5);

    vec3 thickness = vec3(0.0);

    for(int j = 0; j < steps; ++j, position += increment) {
        thickness += calculateAtmosphereDensity(length(position)) * stepSize;
    }

    vec3 transmittance = ec * thickness;

    return max(expf(-transmittance), 0.0);
}

vec3 calculateAtmosphereTransmittance(vec3 rayVector, vec3 position, const int steps) {
    return calculateAtmosphereTransmittance(rayVector, position, extinctionCoefficients, steps);
}

vec3 calculateAtmosphereTransmittance(vec3 rayVector, vec3 position) {
    return calculateAtmosphereTransmittance(rayVector, position, extinctionCoefficients, TRANSMITTANCE_STEPS);
}