
/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

uniform float nightVision;

//This is temporary.

float computeEV100FromAvgLuminance(float avgLuminance) {
    return log2(avgLuminance * 100.0 / 12.5);
}

float convertEV100ToExposure (float EV100) {
    float maxLuminance = 1.2 * pow(2.0, EV100);

    return maxLuminance;
}

float get_luma_perceptual(vec3 x) {     //exposure can get quite an eyesore with primarily warm colors, which is why i decided to use these coefficients since they give much more desirable results
    return dot(x, vec3(0.299, 0.587, 0.114));
}

const float K = 19.5 * exposure_k;   //default: 18.0
const float calibration = exp2(-5.0) * K / 100.0;

const float minExposure = exp2(-5.0) * pi /  dot(sunIlluminance, vec3(0.299, 0.587, 0.114)) * 1.5;
float maxExposure = exp2(-5.0) * pi / (get_luma_perceptual(moonIlluminance) * 5.0 * (1 - ceil(nightVision) * 0.75));

float computeCameraExposure() {
    vec3 averageSamples = textureLod(colortex0, vec2(0.5), ceil(log2(max(viewSize.x, viewSize.y)) * 1.5)).rgb;

    float averageBrightness = rcp(get_luma_perceptual(averageSamples));

    float autoEV100 = computeEV100FromAvgLuminance(clamp(calibration * averageBrightness, minExposure, maxExposure));
    float currentEV = autoEV100;
    float exposure = convertEV100ToExposure(currentEV);
    float exposureTime = frameTime * rcp(mix(3.5, 0.35, float(exposure < texture(colortex5, vec2(0.0) + pixelSize * 0.5).a)) + frameTime * rcp(exposure_speed));
    exposure = mix(texture(colortex5, vec2(0.0) + pixelSize * 0.5).a, exposure, exposureTime);

    return max(exposure, 0.0);
}