
/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/


float dist_beckmann(float ndoth, float alpha2) {
    ndoth *= ndoth;
    float e = exp((ndoth - 1.0) / (alpha2 * tan(ndoth)));
    float num = 1.0 / (pi * alpha2 * cos(ndoth * ndoth));
    return num*e;
}

float dist_trowbridgeReitz(float nDotH, float alpha2) {
        nDotH  *= nDotH;
    float denom     = nDotH * (alpha2 - 1.0) + 1.0;

    return max(alpha2, 1e-5) * rcp(pi * sqr(denom));
}

float smithBeckmann(float cosTheta, float alpha) {
	float c = cosTheta / (alpha * sqrt(1.0 - cosTheta * cosTheta));

	if (c >= 1.6) {
		return 1.0;
	} else {
        float c2 = c * c;
        return (3.535 * c + 2.181 * c2) / (1.0 + 2.276 * c + 2.577 * c2);
    }
}

float smithBeckmannShadowing(float ndotv, float ndotl, float a) {
    return smithBeckmann(ndotl, a) * smithBeckmann(ndotv, a);
}

vec3 brdf_specular(in vec3 view, in vec3 normal, in vec3 light, pbr_data pbr) {
    vec3 halfway = view + light;

    float h_norm = inversesqrt(dot(halfway, halfway));

    float nDotL = (dot(normal, light));
    float vDotN = (dot(view, normal))*h_norm;
    float vDotH = (dot(view, halfway))*h_norm;
    float nDotH = (dot(normal, halfway))*h_norm;

    vec3 F = get_fresnel(pbr.eta, pbr.etak, vDotH, pbr.is_metal);

    float D = 0.0;
        if (pbr.is_metal) D = dist_trowbridgeReitz(nDotH, pbr.roughnessSquared);
        else D = dist_beckmann(nDotH, max(pbr.roughnessSquared, 2e-5));

    float G = smithBeckmannShadowing(vDotN, nDotL, pbr.roughnessSquared);

	vec3 numerator   = F * D * G;
	float denominator = 4.0 * vDotN * nDotL;

	return max(numerator / denominator, 0.0);
}