
/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/



float sqr(in float x) {
    return _sqr(x);
}

int sqr(in int x) {
    return _sqr(x);
}

vec2 sqr(in vec2 x) {
    return _sqr(x);
}

vec3 sqr(in vec3 x) {
    return _sqr(x);
}

vec4 sqr(in vec4 x) {
    return _sqr(x);
}

float cube(float x) { return x * x * x; }
vec2 cube(vec2 x) { return x * x * x; }
vec3 cube(vec3 x) { return x * x * x; }
vec4 cube(vec4 x) { return x * x * x; }

int pow5(in int x) {
    return _pow5(x);
}

vec2 pow5(in vec2 x) {
    return _pow5(x);
}

vec3 pow5(in vec3 x) {
    return _pow5(x);
}

vec4 pow5(in vec4 x) {
    return _pow5(x);
}

float pow3(in float x) {
    return sqr(x)*x;
}

float pow4(in float x) {
    return sqr(x)*sqr(x);
}

float pow5(in float x) {
    return pow4(x)*x;
}

float pow6(in float x) {
    return pow5(x)*x;
}

float pow8(in float x) {
    return pow4(x)*pow4(x);
}

float saturate(in float x) {
    return _saturate(x, 0.0, 1.0);
}

int saturate(in int x) {
    return _saturate(x, 0, 1);
}

vec2 saturate(in vec2 x) {
    return _saturate(x, 0.0, 1.0);
}

vec3 saturate(in vec3 x) {
    return _saturate(x, 0.0, 1.0);
}

vec4 saturate(in vec4 x) {
    return _saturate(x, 0.0, 1.0);
}

float cubic_smooth(in float x) {
    float y = 3.0, z = 2.0;
    return _cubic_smooth(x, y, z);
}

int cubic_smooth(in int x) {
    int y = 3, z = 2;
    return _cubic_smooth(x, y, z);
}

vec2 cubic_smooth(in vec2 x) {
    vec2 y = vec2(3.0), z = vec2(2.0);
    return _cubic_smooth(x, y, z);
}

vec3 cubic_smooth(in vec3 x) {
    vec3 y = vec3(3.0), z = vec3(2.0);
    return _cubic_smooth(x, y, z);
}

vec4 cubic_smooth(in vec4 x) {
    vec4 y = vec4(3.0), z = vec4(2.0);
    return _cubic_smooth(x, y, z);
}

float rcp(in float x) {
    return _rcp(x, 1.0);
}

float rcp(in uint x) {
    return _rcp(float(x), 1.0);
}
float rcp(in int x) {
    return _rcp(x, 1.0);
}

vec2 rcp(in vec2 x) {
    return _rcp(x, 1.0);
}

vec3 rcp(in vec3 x) {
    return _rcp(x, 1.0);
}

vec4 rcp(in vec4 x) {
    return _rcp(x, 1.0);
}

float log10(in float x) {
    return _log10(x, 10.0);
}

int log10(in int x) {
    return int(_log10(x, 10.0));
}

vec2 log10(in vec2 x) {
    return _log10(x, 10.0);
}

vec3 log10(in vec3 x) {
    return _log10(x, 10.0);
}

vec4 log10(in vec4 x) {
    return _log10(x, 10.0);
}

float max3(float x, float y, float z) {
    return max(x, max(y, z));
}

float max3(vec3 x) {
    return max(x.x, max(x.y, x.z));
}

float min3(float x, float y, float z) {
    return min(x, min(y, z));
}

float min3(vec3 x) {
    return min(x.x, min(x.y, x.z));
}

float acosFast(float x) {
    float y     = abs(x);

    float r     = -0.156583 * y + (0.5 * pi);
        r      *= sqrt(1.0 - x);

    return (x >= 0) ? r : pi - r; 
}

//MAD optimized smoothsteps
float sstep(float x, float low, float high){
    float t     = saturate(x * rcp(high - low) - (low * rcp(high - low)));
    return sqr(t) * (t * -2.0 + 3.0);
}
vec2 sstep(vec2 x, vec2 low, vec2 high){
    vec2 t      = saturate(x * rcp(high - low) - (low * rcp(high - low)));
    return sqr(t) * (t * -2.0 + 3.0);
}
vec3 sstep(vec3 x, vec3 low, vec3 high){
    vec3 t      = saturate(x * rcp(high - low) - (low * rcp(high - low)));
    return sqr(t) * (t * -2.0 + 3.0);
}
vec4 sstep(vec4 x, vec4 low, vec4 high){
    vec4 t      = saturate(x * rcp(high - low) - (low * rcp(high - low)));
    return sqr(t) * (t * -2.0 + 3.0);
}

//MAD optimized linear step / remap
float linStep(float x, float low, float high) {
    return saturate(x * rcp(high - low) - (low * rcp(high - low)));
}
vec2 linStep(vec2 x, vec2 low, vec2 high) {
    return saturate(x * rcp(high - low) - (low * rcp(high - low)));
}
vec3 linStep(vec3 x, vec3 low, vec3 high) {
    return saturate(x * rcp(high - low) - (low * rcp(high - low)));
}
vec4 linStep(vec4 x, vec4 low, vec4 high) {
    return saturate(x * rcp(high - low) - (low * rcp(high - low)));
}

vec3 greaterThanVec3(vec3 x, vec3 y, vec3 a, vec3 b) {
    vec3 data   = vec3(0.0);
        data.x  = x.x > y.x ? a.x : b.x;
        data.y  = x.y > y.y ? a.y : b.y;
        data.z  = x.z > y.z ? a.z : b.z;

    return data;
}
vec3 smallerThanVec3(vec3 x, vec3 y, vec3 a, vec3 b) {
    vec3 data   = vec3(0.0);
        data.x  = x.x < y.x ? a.x : b.x;
        data.y  = x.y < y.y ? a.y : b.y;
        data.z  = x.z < y.z ? a.z : b.z;

    return data;
}

float v3avg(vec3 x) {
    return (x.x+x.y+x.z)/3.0;
}

//only use these on scenecolor or taa buffer
vec4 make_drawbuffer(in vec3 scenecol, in float alpha) {
    #ifdef MC_GL_RENDERER_GEFORCE
        vec3 temp   = clamp(scenecol, 1.0/65530.0, 65535.0);   //NaN fix on nvidia
    #else
        vec3 temp   = clamp(scenecol, 0.0, 65535.0);
    #endif

    return vec4(temp, alpha);
}
vec4 make_drawbuffer(in vec3 scenecol) {
    #ifdef MC_GL_RENDERER_GEFORCE
        vec3 temp   = clamp(scenecol, 1.0/65530.0, 65535.0);   //NaN fix on nvidia
    #else
        vec3 temp   = clamp(scenecol, 0.0, 65535.0);
    #endif

    return vec4(temp, 1.0);
}
vec4 make_drawbuffer(in vec4 scenecol) {
    #ifdef MC_GL_RENDERER_GEFORCE
        vec3 temp   = clamp(scenecol.rgb, 1.0/65530.0, 65535.0);
    #else
        vec3 temp   = clamp(scenecol.rgb, 0.0, 65535.0);
    #endif

    return vec4(temp, max(scenecol.a, 0.0));
}

//these are for non-scenecolor stuffs
vec4 clamp_drawbuffer(in vec3 scenecol) {
    vec3 temp   = clamp(scenecol, 0.0, 65535.0);

    return vec4(temp, 1.0);
}
vec4 clamp_drawbuffer(in vec4 scenecol) {
    vec3 temp   = clamp(scenecol.rgb, 0.0, 65535.0);

    return vec4(temp, max(scenecol.a, 0.0));
}

vec2 rsi(vec3 pos, vec3 dir, float radius) {
    float radiusSquared = sqr(radius);
    float posDotDir = dot(pos, dir);
    float endDist = posDotDir*posDotDir + radiusSquared - length2(pos);

    if(endDist < 0.0) return vec2(-1.0);

    endDist = sqrt(endDist);
    vec2 ret = -posDotDir + vec2(-endDist, endDist);

    return ret;
}

vec2 circleMap(in float point) {
    return vec2(cos(point), sin(point));
}

vec2 sincos(in float x) {
    return vec2(sin(x), cos(x));
}

float get_luma(vec3 x) {
    return dot(x, lumacoeffAP1);
}