
/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/



#include "/lib/universal.glsl"

//bloom downsampling method based on chocapic13's shaders
//merge and upsample blurs

/* ------ framebuffer attachment outputs ------ */
/*DRAWBUFFERS:4*/
layout(location = 0) out vec4 blur_data;

/* ------ inputs from vertex stage ------ */
in vec2 coord;

/* ------ uniforms ------ */
uniform vec2 pixelSize;
uniform vec2 viewSize;

uniform sampler2D colortex4;

/* ------ includes ------ */
#include "/lib/util/bicubic.glsl"

void main() {
    vec2 tcoord     = (gl_FragCoord.xy*2.0+0.5)*pixelSize;
    vec2 rscale     = vec2(1920.0, 1080.0)/max(viewSize, vec2(1920.0, 1080.0));

    blur_data.rgb   = textureBicubic(colortex4, (tcoord+vec2(0.0, 0.5))/2.0).rgb;                                // 1:4
    blur_data.rgb  += textureBicubic(colortex4, tcoord/4.0).rgb;                                                 // 1:8
    blur_data.rgb  += textureBicubic(colortex4, tcoord/8.0+vec2(0.25*rscale.x+2.0*pixelSize.x, 0.0)).rgb;        // 1:16
    blur_data.rgb  += textureBicubic(colortex4, tcoord/16.0+vec2(0.375*rscale.x+4.0*pixelSize.x, 0.0)).rgb;      // 1:32
    blur_data.rgb  += textureBicubic(colortex4, tcoord/32.0+vec2(0.4375*rscale.x+6.0*pixelSize.x, 0.0)).rgb;     // 1:64
    blur_data.rgb  += textureBicubic(colortex4, tcoord/64.0+vec2(0.46875*rscale.x+8.0*pixelSize.x, 0.0)).rgb;    // 1:128
    blur_data.rgb  += textureBicubic(colortex4, tcoord/128.0+vec2(0.484375*rscale.x+10.0*pixelSize.x, 0.0)).rgb; // 1:256

    blur_data.rgb  *= 0.5;

    blur_data       = clamp_drawbuffer(blur_data.rgb);

}