/*
====================================================================================================
    __  __    __  ___    ____    ____
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/
    /_/   /_/  /_/   /___/   /_/ |_|

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/

float value3D(in vec3 pos) {
    vec3 p = floor(pos);
    vec3 b = fract(pos);

    vec2 uv = (p.xy + vec2(-97.0) * p.z) + b.xy;
    vec2 rg = texture(noisetex, uv / 256.0).xy;

    float ret = mix(rg.x, rg.y, b.z);
    return cubic_smooth(ret) * 2.0 - 1.0;
}

vec2 rotateCoord(in vec2 pos, in const float angle) {
    return vec2(cos(angle) * pos.x + sin(angle) * pos.y,
                cos(angle) * pos.y - sin(angle) * pos.x);
}

float windTick = frameTimeCounter * pi;

float windMag(in float p, in float speed, in float soft) {
    return speed * windTick + p * soft * wf_softness;
}

float speedWave(in vec2 pos, in const float initial) {
    #ifdef dynamic_wind_direction
        pos = rotateCoord(pos, windRotation);
    #endif
    
    float x1 = windMag(pos.x + pos.y, 0.17 * wf_speed_wave_period, 0.075);
    float x2 = windMag(pos.x + pos.y, 0.11 * wf_speed_wave_period, 0.075);
    return initial + (sin(x1) + cos(x2)) * 0.5;
}

vec2 windMicro(in vec3 pos) {
    vec2 micro = vec2(0.0);
    micro += value3D(pos * 2.7 + vec3(0.52, 0.34, 0.37) * windTick) * vec2(0.9,  0.7);
    micro += value3D(pos * 3.2 + vec3(0.42, 0.35, 0.40) * windTick) * vec2(0.7,  0.8);
    micro -= value3D(pos * 2.4 + vec3(0.37, 0.49, 0.55) * windTick) * vec2(0.6, -0.5);
    micro -= value3D(pos * 4.1 + vec3(0.31, 0.44, 0.39) * windTick) * vec2(0.8, -0.3);

    float x = windMag(pos.x + pos.z, 0.1, 0.15);
    micro *= max(sin(x) + 0.6, 0.0) * 0.5 + 0.2;

    return micro;
}

vec2 windOffset(in vec3 pos) {
    pos.xz       = rotateCoord(pos.xz, pi * rcp(3.0));
    vec2 offset  = vec2(0.0);
    vec2 dir     = vec2(1.0, 0.2);
    float speed  = 1.0;

    #ifdef wf_macro
        float x1 = windMag(pos.x + pos.y * 0.15 + pos.z, 1.25, 4.0); // macro
        offset  += (sin(x1) + 0.2) * dir * wf_macro_strength;
    #endif

    #ifdef wf_meso
        float x2 = windMag(pos.x + pos.y * 0.30 + pos.z, 1.0, 1.2); // meso
        float x3 = windMag(pos.x + pos.y * 0.30 + pos.z, 0.6, 1.2); // reverse
        offset  += (sin(x2) + cos(x3)) * vec2(dir.x, -dir.y) * wf_meso_strength;
    #endif
    
    #ifdef wf_speed_wave
        speed    = speedWave(pos.xz, 1.1);
    #endif

    if(rainStrength != 0.0) speed = speed / (rainStrength + 1.0) + rainStrength * 1.5;
    offset      *= speed;
    
    #ifdef dynamic_wind_direction
        offset   = rotateCoord(offset, windRotation);
    #endif

    #ifdef wf_micro
        offset  += windMicro(pos) * wf_micro_strength;
    #endif

    return offset * 0.135;
}

vec3 windFoliage(in vec3 pos) {
    bool topvert = gl_MultiTexCoord0.t < mc_midTexCoord.t,
         leaves = mc_Entity.x == 10021, 
         small_foliage = mc_Entity.x == 10022 && topvert,
         saplings = mc_Entity.x == 10023 && topvert, 
         tall_foliage_lower = mc_Entity.x == 10024 && topvert, 
         tall_foliage_upper = mc_Entity.x == 10025;
    
    if(leaves || small_foliage || saplings || tall_foliage_lower || tall_foliage_upper) {
        vec2 offset = windOffset(pos.xyz + cameraPosition);

        if(leaves) {
            pos.xyz += offset.xyy * 0.2;
        } else if(saplings || tall_foliage_lower || (tall_foliage_upper && !topvert)) {
            pos.xz += offset * 0.5;
        } else {
            pos.xz += offset;
        }
    }

    return pos;
}
