/*
temporal anti aliasing based on
- bsl shaders
- chocapic13 shaders
- unreal 4
*/



#include "/lib/universal.glsl"

/* ------ framebuffer attachment outputs ------ */
/*DRAWBUFFERS:025*/
layout(location = 0) out vec4 scenecol_data;
layout(location = 1) out vec4 rain_data;
layout(location = 2) out vec4 temporal_data;

/* ------ inputs from vertex stage ------ */
in vec2 coord;

/* ------ uniforms ------ */
uniform float frameTime;
uniform float rainStrength;

uniform vec2 viewSize, pixelSize;

uniform vec3 cameraPosition, previousCameraPosition;

uniform mat4 gbufferPreviousModelView, gbufferPreviousProjection;
uniform mat4 gbufferProjectionInverse, gbufferModelViewInverse;

uniform sampler2D colortex0;
uniform sampler2D colortex2;
uniform sampler2D colortex5;
uniform sampler2D depthtex1;


/* ------ includes ------ */
#include "/lib/util/encoders.glsl"

/*
vec4 reproject(vec4 position) {
	position = position * 2.0 - 1.0;

	position      = gbufferModelViewInverse * gbufferProjectionInverse * position;
	position     /= position.w;
	position.xyz += cameraPosition;

	position.xyz -= previousCameraPosition;
	position      = gbufferPreviousProjection * gbufferPreviousModelView * position;
	position     /= position.w;

	position = position * 0.5 + 0.5;
	return position;
}*/

vec2 reproject(vec2 coord, float depth) {
    float hand  	= float(depth > 0.56);

    vec4 position	= vec4(coord, depth, 1.0)*2.0-1.0;
        position    = gbufferProjectionInverse*position;
        position   /= position.w;
        position    = gbufferModelViewInverse*position;

		position   += vec4(cameraPosition-previousCameraPosition, 0.0)*hand;
		position    = gbufferPreviousModelView*position;
		position    = gbufferPreviousProjection*position;

    return (position.xy/position.w)*0.5+0.5;
}

vec4 texture_catmullrom(sampler2D tex, vec2 uv) {
    vec2 res    = textureSize(tex, 0);

    vec2 coord  = uv*res;
    vec2 coord1 = floor(coord - 0.5) + 0.5;

    vec2 f      = coord-coord1;

    vec2 w0     = f*(-0.5 + f*(1.0-0.5*f));
    vec2 w1     = 1.0 + sqr(f)*(-2.5+1.5*f);
    vec2 w2     = f*(0.5 + f*(2.0-1.5*f));
    vec2 w3     = sqr(f)*(-0.5+0.5*f);

    vec2 w12    = w1+w2;
    vec2 delta12 = w2/w12;

    vec2 uv0    = coord1 - vec2(1.0);
    vec2 uv3    = coord1 + vec2(1.0);
    vec2 uv12   = coord1 + delta12;

        uv0    *= pixelSize;
        uv3    *= pixelSize;
        uv12   *= pixelSize;

    vec4 col    = vec4(0.0);
        col    += textureLod(tex, vec2(uv0.x, uv0.y), 0)*w0.x*w0.y;
        col    += textureLod(tex, vec2(uv12.x, uv0.y), 0)*w12.x*w0.y;
        col    += textureLod(tex, vec2(uv3.x, uv0.y), 0)*w3.x*w0.y;

        col    += textureLod(tex, vec2(uv0.x, uv12.y), 0)*w0.x*w12.y;
        col    += textureLod(tex, vec2(uv12.x, uv12.y), 0)*w12.x*w12.y;
        col    += textureLod(tex, vec2(uv3.x, uv12.y), 0)*w3.x*w12.y;

        col    += textureLod(tex, vec2(uv0.x, uv3.y), 0)*w0.x*w3.y;
        col    += textureLod(tex, vec2(uv12.x, uv3.y), 0)*w12.x*w3.y;
        col    += textureLod(tex, vec2(uv3.x, uv3.y), 0)*w3.x*w3.y;

    return clamp(col, 0.0, 65535.0);
}

#define taa_blend 0.2
#define taa_mreject 0.5
#define taa_antighost 1.0
#define taa_antiflicker 0.6

vec3 temporalAntiAliasing(vec3 scenecol) {
    vec3 returnColor 	= vec3(0.0);
	float depth 		= stex(depthtex1).r;
	vec2 reprojectedPos = reproject(coord, depth);

	float blendWeight = 0.9;
	/*
	const float offcenter_rejection = 0.5;
	vec2 pixelCenterDist = 1.0 - abs(2.0 * fract(reprojectedPos * viewSize) - 1.0);
	blendWeight *= sqrt(pixelCenterDist.x * pixelCenterDist.y) * offcenter_rejection + (1.0 - offcenter_rejection);
	blendWeight  = max(blendWeight, 0.35);

	if (!landMask(depth)) blendWeight = max(blendWeight, 0.9);

	if (clamp(reprojectedPos, 0.0, 1.0) != reprojectedPos) { blendWeight *= 0.0; }

	vec4 currentFrame = texture(colortex0, coord);
	vec4 previousFrame = vec4(texture(colortex5, reprojectedPos.xy).rgb, 1.0);*/

	vec2 px_dist    = 0.5-abs(fract((reprojectedPos-coord)*viewSize)-0.5);

    float blend_weight	= dot(px_dist, px_dist);
        blend_weight    = pow(blend_weight, 1.5)*taa_mreject;

    if (clamp(reprojectedPos, 0.0, 1.0) != reprojectedPos) return scenecol;

	vec3 tl = textureLod(colortex0,coord+vec2(-pixelSize.x, -pixelSize.y),0).rgb;
	vec3 tc = textureLod(colortex0,coord+vec2( 0.0,         	-pixelSize.y),0).rgb;
	vec3 tr = textureLod(colortex0,coord+vec2( pixelSize.x, -pixelSize.y),0).rgb;
	vec3 ml = textureLod(colortex0,coord+vec2(-pixelSize.x, 0.0         	),0).rgb;
	vec3 mr = textureLod(colortex0,coord+vec2( pixelSize.x, 0.0         	),0).rgb;
	vec3 bl = textureLod(colortex0,coord+vec2(-pixelSize.x,  pixelSize.y),0).rgb;
	vec3 bm = textureLod(colortex0,coord+vec2( 0.0,          	 pixelSize.x),0).rgb;
	vec3 br = textureLod(colortex0,coord+vec2( pixelSize.x,  pixelSize.y),0).rgb;

	vec3 min_col = min(min(min(min(min(min(min(min(tl, tc), tr), ml), scenecol), mr), bl), bm), br);
	vec3 max_col = max(max(max(max(max(max(max(max(tl, tc), tr), ml), scenecol), mr), bl), bm), br);

	//returnColor.rgb = mix(currentFrame.rgb, clamp(previousFrame.rgb, minCol, maxCol), blendWeight);

	vec3 reprojection	= texture_catmullrom(colortex5, reprojectedPos).rgb;

	vec3 taa_color 		= clamp(reprojection, min_col, max_col);

	float clamped 		= distance(reprojection, taa_color)/get_luma(reprojection);

    //flicker reduction
    float lum_diff      = distance(reprojection, scenecol)/get_luma(reprojection);
        lum_diff        = 1.0-saturate(sqr(lum_diff))*taa_antiflicker;

    vec2 velocity       = (coord-reprojectedPos)/pixelSize;

    float taa_weight 	= saturate(1.0-sqrt(length(velocity))/2.0)*0.9;

    if (landMask(depth)) taa_weight = max(taa_weight, 0.05);
    else taa_weight = max(taa_weight, 0.35);

    float lb    = taa_blend;
    if (!landMask(depth)) lb = 0.1;

    taa_weight  = mix(taa_weight, 0.982, 1.0-saturate(lum_diff*lb + blend_weight + clamped*taa_antighost));

	return mix(scenecol.rgb, taa_color.rgb, taa_weight);
}

const float gauss9w[9] = float[9] (
     0.0779, 0.12325, 0.0779,
    0.12325, 0.1954,  0.12225,
     0.0779, 0.12325, 0.0779
);

const vec2 gauss9o[9] = vec2[9] (
    vec2(1.0, 1.0), vec2(0.0, 1.0), vec2(-1.0, 1.0),
    vec2(1.0, 0.0), vec2(0.0, 0.0), vec2(-1.0, 0.0),
    vec2(1.0, -1.0), vec2(0.0, -1.0), vec2(-1.0, -1.0)
);

float gauss9_rain(sampler2D tex) {
    float col        = 0.0;

    for (int i = 0; i<9; i++) {
        vec2 bcoord = coord + gauss9o[i]*pixelSize;
        col += float(decodeMatID16(texture(tex, bcoord).x) == 301)*gauss9w[i];
    }
    return col;
}

void main() {
    scenecol_data     = stexLod(colortex0, 0);

    #ifdef taa_enabled
    vec3 taa          = temporalAntiAliasing(scenecol_data.rgb);
    scenecol_data.rgb = taa;
    #else
    vec3 taa        = scenecol_data.rgb;
    #endif

    rain_data       = vec4(stex(colortex2).x, 0.0, 0.0, 1.0);

    if (rainStrength > 0.0) rain_data.y = gauss9_rain(colortex2);
    else rain_data.y = 0.0;

    scenecol_data   = make_drawbuffer(scenecol_data);
    temporal_data   = clamp_drawbuffer(vec4(taa, stex(colortex5).a));
}