
/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/



/* --- internals --- */
#define SKY_RENDER_LOD 2
#define SKYREF_LOD 3.0
#define vcloud_shadow_clip 5e4

/* --- debugging --- */
//#define hide_terrain
//#define white_world

/* --- atmosphere customization --- */
#define SCATTERING_STEPS 24     //[16 24 32 40 48 56 64 128 256]
#define TRANSMITTANCE_STEPS 12  //[8 10 12 14 16 18 20 22 24 28 32 64 128]
#define atmos_mieGmult  1.0     //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]
#define atmos_mieCoeff  1.0     //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.2 2.4 2.6 2.8 3.0]
#define atmos_rayCoeff  1.0     //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.4 1.6 1.8 2.0 2.2 2.4 2.6 2.8 3.0 4.0 5.0 6.0 7.0 8.0]
#define rainbows_enabled

/* --- volumetric fog --- */
#define vfog_enabled
#define vfog_quality 1.0    //[0.2 0.4 0.6 0.8 1.0 1.2 1.4 1.6 1.8 2.0]
#define vfog_bilateral
#define vfog_alt 70.0       //[50.0 60.0 70.0 80.0 90.0 100.0 110.0 120.0 130.0 140.0 150.0 175.0 200.0]
#define vfog_mie_density 1.0    //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.2 1.4 1.6 1.8 2.0 2.2 2.4 2.6 2.8 3.0 3.5 4.0 4.5 5.0 6.0 7.0 8.0 9.0 10.0]
#define vfog_rayl_density 1.0    //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.2 1.4 1.6 1.8 2.0 2.2 2.4 2.6 2.8 3.0 3.5 4.0 4.5 5.0 6.0 7.0 8.0 9.0 10.0]
#define vfog_mist_density 1.0    //[0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.2 1.4 1.6 1.8 2.0 2.2 2.4 2.6 2.8 3.0 3.5 4.0 4.5 5.0 6.0 7.0 8.0 9.0 10.0]
#define vfog_maxdist 512.0  //[128.0 192.0 256.0 384.0 512.0 768.0 1024.0 1536.0 2048.0]

#define vcloud_enabled
#define vcloud_resolution 0     //[0 1 2]
//#define vcloud_ditheredLight
#ifdef vcloud_enabled
#define cloud_shadows
#endif


#if vcloud_resolution == 0
    const float CLOUD_RENDER_LOD = 4.0;
#elif vcloud_resolution == 1
    const float CLOUD_RENDER_LOD = 3.0;
#elif vcloud_resolution == 2
    const float CLOUD_RENDER_LOD = 2.0;
#endif


/* --- dynamics --- */
#define atmos_dynamic
#define vcloud_dynamic_type
#define vfog_dynamic_density

/* --- lighting stuffs --- */
#define shadowFarPlaneMult 0.2  //[0.1 0.2 0.3 0.4 0.5]
#define shadowBiasMult 1.0  //[0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.4 1.6 1.8 2.0 2.5 3.0]
#define shadow_vps_enabled
#define shadow_vps_iterations 6     //[4 5 6 7 8 9 10]
#define ambient_occlusion_enabled
#define ibl_mode 1          //[0 1 2 3] Image-based lighting
#define ibl_iterations 12    //[8 10 12 14 16 20 24 28 32]
//#define rsm_enabled
#define rsm_steps 20        //[4 8 12 16 20 24 32 48 64]
#define rsm_size 40.0       //[5.0 10.0 20.0 30.0 40.0 50.0 60.0 80.0 100.0]

#define blocklightTemp 2200.0   //[1000.0 1200.0 1400.0 1600.0 1800.0 2000.0 2200.0 2400.0 2600.0 2800.0 3000.0 3200.0 3400.0 3600.0 3800.0 4000.0 4200.0 4400.0 4600.0 4800.0 5000.0]
#define blocklightMult 1.0 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.5 3.0 3.5 4.0 5.0 6.0 7.0 8.0 9.0 10.0 15.0 20.0 25.0 30.0 35.0 40.0 50.0 75.0 100.0 150.0 200.0]


/* --- resourcepack related --- */
#define texture_format 13    //[13 12 11]
//#define normals_resourcepack
//#define specular_resourcepack
#define hardcoded_specular //Toggle hardcoded (resource pack independent) specularity of gold and iron blocks. "Resource Pack Specular" must be disabled!

/* --- reflections --- */
#define reflections_enabled
#define ssr_enabled
#define cloud_reflections_enabled

/* --- world stuff --- */
//#define water_normal_hq
#define dynamic_wind_direction // Makes the wind change direction over time. Affects foliage and rain.

/* --- waving foliage --- */
#define waving_foliage_enabled
#define wf_softness 1.0        // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 2.8 2.9 3.0]
#define wf_lod 175.0        // [50.0 75.0 100.0 125.0 150.0 175.0 200.0 225.0 250.0]
#define wf_shadow_lod 100.0 // [50.0 75.0 100.0 125.0 150.0 175.0 200.0 225.0 250.0]
#define wf_macro                // Main wind direction and movement.
#define wf_meso                 // Medium wind movement like gusts.
#define wf_micro                // Small, random wind movement.
#define wf_macro_strength 0.6   // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 2.8 2.9 3.0]
#define wf_meso_strength 0.4    // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 2.8 2.9 3.0]
#define wf_micro_strength 0.4   // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 2.8 2.9 3.0]
#define wf_speed_wave
#define wf_speed_wave_period 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 2.8 2.9 3.0] "Stretches/squeezes" the speed wave.

/* --- post process effects -- */
#define taa_enabled
#define tonemap academy_color_encoding_system //[academy_color_encoding_system academy_color_encoding_system_rrt hable]

/* --- lens effects --- */
#define cameraSensorWidth 35.0      //[25.0 30.0 35.0 40.0 50.0 60.0 70.0 80.0]
#define cameraFStop 2.8           //[1.4 2.0 2.8 4.0 5.6 8.0 11.0 16.0 22.0 32.0] //The ratio of focal length to diaphram size, for more precise numbers go to shaders/settings.glsl and change this number manually.
#define cameraAnamorphDistortion 1.0 //[0.1 0.2 0.3 0.4 0.5 0.55 0.6 0.65 0.7 0.75 0.8 0.85 0.9 0.95 1.0 1.2 1.25 1.3 1.35 1.4 1.45 1.5 1.6 1.7 1.8 1.9 2.0]
#define cameraBarrelDistortion 0.0 //[0.0 0.05 0.1 0.15 0.2 0.25 0.3 0.35 0.4 0.45 0.5 0.55 0.6 0.65 0.7 0.75 0.8 0.85 0.9 0.95 1.0]

#define LensLOD 2.0 //[0.0 1.0 2.0 3.0 4.0]

#define BLADES 5 //[4 5 6 7 8 16] //Number of blades in the aperture. This number is not limited, for more numbers go to lib/settings.glsl and change this number manually.
#define ROTATION 0.0 //[0.0 0.1 0.15 0.2 0.25 0.3 0.35 0.4 0.45 0.5 0.55 0.6 0.65 0.7 0.75 0.8 0.85 0.9 0.95 1.0] //How much the aperture is rotated. 0.0 has no rotation.
#define ROUNDING 0.0 //[0.0 0.1 0.15 0.2 0.25 0.3 0.35 0.4 0.45 0.5] //How much the aperture is rounded. 0.0 is a perfect polygon, 1.0 is a perfect circle.
#define LENS_SHIFT_AMOUNT 0.1 //[0.0 0.1 0.15 0.2 0.25 0.3 0.35 0.4 0.45 0.5] //The factor for how much light is split outwards. 0.0 has no splitting mimicking a very high quality glass, 0.5 is an unrealistic ammount of splitting.

/* dof */
//#define dof_enabled
#define dofSamples 40 //[16 18 20 22 24 26 28 30 32 34 36 38 40 42 44 46 48 50 52 54 56 58 60 62 64 68 70 72 74 76 78 80 82 84 86 88 90 92 94 96 98 100 102 104 106 108 110 112 114 116 118 120 122 124 126 128]
#define dofLodEnabled
//#define dof_exclude_hand

#define BokehSoftness 0.9

/* motionblur */
#define motionblur_enabled
#define motionblur_samples 8    //[4 6 8 10 12 14 16]
#define motionblur_size 1.0     //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]

/* film grain */
#define filmgrain_enabled
#define filmgrain_strength 1.0  //[0.5 1.0 2.0 3.0 4.0 5.0 6.0 7.0 8.0 9.0 10.0]

/* post stuffs */
#define exposure_k 1.0    //[0.5 0.55 0.6 0.65 0.7 0.75 0.8 0.85 0.9 0.95 1.0 1.2 1.25 1.3 1.35 1.4 1.45 1.5]
#define exposure_speed 1.0 //[0.5 0.6 0.7 0.8 0.9 1.0 1.2 1.4 1.6 1.8 2.0 2.5 3.0]
//#define manual_exposure_enabled
#define manual_exposure 300.0     //[0.1 0.3 0.5 1.0 1.5 2.0 3.0 4.0 5.0 6.0 7.0 8.0 9.0 10.0 12.0 14.0 16.0 18.0 20.0 25.0 30.0 50.0 75.0 100.0 150.0 200.0 250.0 300.0 350.0 400.0 450.0 500.0]

/* misc */
#define bloom_enabled

/* --- constants --- */
const vec2 bokehOffset = vec2(0.5);