#version 450 compatibility

/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/



#include "/lib/universal.glsl"

/* ------ outputs ------ */
out mat2x2 coord;   //0-texcoord, 1-lightmap

out vec4 tint;

/* ------ uniforms & attributes ------ */
#ifdef dynamic_wind_direction
    uniform float windRotation;
#endif

uniform float frameTimeCounter;

uniform vec2 taaOffset;

uniform vec3 sunVector, sunVectorView;
uniform vec3 moonVector, moonVectorView;
uniform vec3 sunPosition, upPosition;

uniform vec3 cameraPosition;

uniform mat4 gbufferModelView, gbufferModelViewInverse;

vec2 rotateCoord(in vec2 pos, in const float angle) {
    return vec2(cos(angle) * pos.x + sin(angle) * pos.y,
                cos(angle) * pos.y - sin(angle) * pos.x);
}

float windNoise(vec3 pos) {
    pos    += cameraPosition;

    pos    *= pi;
    pos    *= vec3(0.4, 0.5, 0.4);

    float p     = pos.x + pos.y + pos.z;

    float s1    = sin(p + frameTimeCounter * 0.1) * 0.5 + 0.5;
    float c1    = cos(p + frameTimeCounter * 0.06) * 0.5 + 0.5;

    return (s1 * 0.7 + sqr(c1) * 0.5) * 0.4;
}

void main() {
    coord[0]    = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy * vec2(3.0, 1.4);
    coord[1]    = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    coord[1].y  = linStep(coord[1].y, 1.0/16.0, 1.0);
    tint        = gl_Color;

    vec4 pos    = gl_Vertex;
        pos     = transMAD(gl_ModelViewMatrix, pos.xyz).xyzz;
        pos     = transMAD(gbufferModelViewInverse, pos.xyz).xyzz;

    float wind_mult = windNoise(pos.xyz);
        pos.x  += (pos.y) * wind_mult;

    #ifdef dynamic_wind_direction
        pos.xz  = rotateCoord(pos.xz, windRotation);
    #endif

        pos     = transMAD(gbufferModelView, pos.xyz).xyzz;

    gl_Position = pos.xyzz * diagonal4(gl_ProjectionMatrix) + vec4(0, 0, gl_ProjectionMatrix[3].z, 0);
    gl_Position.xy += taaOffset * gl_Position.w;
}