#version 450 compatibility

/*
====================================================================================================
    __  __    __  ___    ____    ____ 
    \ \/ /   /  |/  /   /  _/   / __ \
     \  /   / /|_/ /    / /    / /_/ /
     / /   / /  / /   _/ /    / _, _/ 
    /_/   /_/  /_/   /___/   /_/ |_|  

    YMIR SHADERS

    Copyright (C) 2020 Jessie, RRe36, Niemand

    All Rights Reserved unless otherwise explicitly stated.


    By downloading this you have agreed to the license and terms of use.
    These can be found inside the included license-file
    or here: https://github.com/ymir-graphics/ymir-releases/blob/master/LICENSE

    Violating these terms may be penalized with actions according to the Digital Millennium
    Copyright Act (DMCA), the Information Society Directive and/or similar laws
    depending on your country.

====================================================================================================
*/




#include "/lib/universal.glsl"

/* ------ outputs ------ */
out mat2x2 coord;

flat out int is_water;

flat out vec3 normal;

flat out vec4 tint;

/* ------ uniforms & attributes ------ */
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;

attribute vec4 mc_Entity;

#ifdef waving_foliage_enabled
    #ifdef dynamic_wind_direction
        uniform float windRotation;
    #endif
    
    uniform float frameTimeCounter;
    uniform float rainStrength;
    
    uniform vec3 cameraPosition;
    
    uniform mat4 shadowModelView;
    uniform mat4 shadowModelViewInverse;
    
    uniform sampler2D noisetex;
    
    attribute vec4 mc_midTexCoord;
#endif

/* ------ includes ------ */
#include "/lib/lighting/warp.glsl"
#ifdef waving_foliage_enabled
    #include "/program/common/foliage.glsl"
#endif

#define transMAD(mat, v) (mat3x3(mat) * (v) + (mat)[3].xyz)

#define diagonal2(mat) vec2((mat)[0].x, (mat)[1].y)
#define diagonal3(mat) vec3(diagonal2(mat), (mat)[2].z)
#define diagonal4(mat) vec4(diagonal3(mat), (mat)[2].w)

#define projMAD3(mat, v) (diagonal3(mat) * (v) + (mat)[3].xyz)

void main() {
    coord[0]    = (gl_TextureMatrix[0]*gl_MultiTexCoord0).xy;
    coord[1]    = (gl_TextureMatrix[1]*gl_MultiTexCoord1).xy;
    coord[1].x  = linStep(coord[1].x, rcp(24.0), 1.0);
    coord[1].y  = linStep(coord[1].y, rcp(16.0), 1.0);

    normal 	= normalize(gl_NormalMatrix*gl_Normal);     //we need this for rsm
    tint    = gl_Color;

    vec4 pos = gl_ModelViewMatrix * gl_Vertex;

    #ifdef waving_foliage_enabled
        pos.xyz = transMAD(shadowModelViewInverse, pos.xyz);
        if(length(pos.xz) < wf_shadow_lod) pos.xyz = windFoliage(pos.xyz);
        pos.xyz = transMAD(shadowModelView, pos.xyz);
    #endif

    //gl_Position = pos.xyzz * diagonal4(gl_ProjectionMatrix) + vec4(0, 0, gl_ProjectionMatrix[3].z, 0);
    gl_Position    = gl_ProjectionMatrix * pos;
    gl_Position.xy = warp_shadowmap(gl_Position.xy);
    gl_Position.z *= shadowFarPlaneMult;   //fuckin far plane on the shadowmap

    //mat ids
    if (mc_Entity.x == 10001) is_water = 1;
    else is_water = 0;
}